function [Y_bus_mic, G_bus_mic, B_bus_mic, P_u_mic, P_l_mic, Q_u_mic, Q_l_mic, V_u_mic, V_l_mic, P_load_mic, Q_load_mic] = model_ini_mg3(switch_mg3)
s_s1s2 = switch_mg3(1);
s_s2s3 = switch_mg3(2);
s_s2s4 = switch_mg3(3);
s_s3s4 = switch_mg3(4);

%Model Initialization Function
%clear all
S_base = 100e3;          %apparent power base value
V_base = 480;         %voltage base value at Bus - 480 V
%impedance base value of three regions%
Z_base = 3*(V_base/sqrt(3))^2/S_base;
%distribution line parameter
R_dist_ln = 0.786;
X_dist_ln = 0.156;
%microgrid line parameter
R_mg_ln = 1.58;
X_mg_ln = 0.175;
%find pu value of each model parameter
Z_s1s2_mic = (R_dist_ln + 1j*X_dist_ln)*0.2/Z_base;
Z_s2s3_mic = (R_mg_ln + 1j*X_mg_ln)*0.2/Z_base;
Z_s2s4_mic = (R_mg_ln + 1j*X_mg_ln)*0.2/Z_base;
Z_s3s4_mic = (R_mg_ln + 1j*X_mg_ln)*0.2/Z_base;

Y_bus_mic = [1/Z_s1s2_mic*s_s1s2        -1/Z_s1s2_mic*s_s1s2                                                0                                               0;
            -1/Z_s1s2_mic*s_s1s2        1/Z_s1s2_mic*s_s1s2+1/Z_s2s3_mic*s_s2s3+1/Z_s2s4_mic*s_s2s4         -1/Z_s2s3_mic*s_s2s3                            -1/Z_s2s4_mic*s_s2s4;
            0                           -1/Z_s2s3_mic*s_s2s3                                                1/Z_s2s3_mic*s_s2s3+1/Z_s3s4_mic*s_s3s4         -1/Z_s3s4_mic*s_s3s4;
            0                           -1/Z_s2s4_mic*s_s2s4                                                -1/Z_s3s4_mic*s_s3s4                            1/Z_s2s4_mic*s_s2s4+1/Z_s3s4_mic*s_s3s4];

G_bus_mic = real(Y_bus_mic);        %conductance matrix G_bus
B_bus_mic = imag(Y_bus_mic);        %susceptance matrix B_bus

V_u_mic = 1.05; V_l_mic = 0.95;
P_u_mic = [0 0.6 0.6 0.6]/3; P_l_mic = [0 0 0 0]/3;
Q_u_mic = [0 0.2 0.2 0.2]/3; Q_l_mic = [0 -0.2 -0.2 -0.2]/3;
P_load_mic = [0 0.3 0.3 0.3]/3; Q_load_mic = [0 0.1 0.1 0.1]/3;