function [sys,x0,str,ts] = opt_solver_mic3_snn(t,x,u,flag)
% Determine Set Point of Microgrid 2
switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes();

  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  case 3,
    sys=mdlOutputs(t,x,u);

  %%%%%%%%%%%%%%%%%%%
  % Unhandled flags %
  %%%%%%%%%%%%%%%%%%%
  case { 1, 2, 4, 9 },
    sys = [];

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    DAStudio.error('Simulink:blocks:unhandledFlag', num2str(flag));

end
% end csfunc

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes()

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 5;
sizes.NumInputs      = 18;
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
x0  = [];
str = [];
ts  = [-1 0];

% end mdlInitializeSizes
%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)
global V_vector_mcm_dec
global V_u_mic3 V_l_mic3 P_u_mic3 P_l_mic3 Q_u_mic3 Q_l_mic3 switch_ln
    V_mag = u(1:4);
    V_pha = u(5:8);

    V_l_mic3 = u(9);
    V_u_mic3 = u(10);
    P_u_mic3 = u(11);
    P_l_mic3 = u(12);
    Q_u_mic3 = u(13);
    Q_l_mic3 = u(14);
    
    trig_ln2 = u(15);
    trig_ln3 = u(16);
    switch_ln = u(17:18);
%initialization of network
[Y_bus_mcm, G_bus_mcm, B_bus_mcm] = model_ini_mcm_mg2(switch_ln);

load obj_mic3_u_temp
load sp_mic3_temp


    V_vector_mcm_dec = [V_mag(1)*exp(1j*V_pha(1)); V_mag(2)*exp(1j*V_pha(2)); V_mag(3)*exp(1j*V_pha(3)); V_mag(4)*exp(1j*V_pha(4))];
    V_vector = V_vector_mcm_dec;

%distributed optimization problem
if ((trig_ln2==1)||(trig_ln3==1)||(abs(u(2) - u_temp(2)) > 0.00001)||(abs(u(4) - u_temp(4)) > 0.00001)||(sp_temp(1) > P_u_mic3+0.00001)||(sp_temp(1)<P_l_mic3-0.00001)||(sp_temp(2) > Q_u_mic3+0.00001)||(sp_temp(2)<Q_l_mic3-0.00001))
    V0_mic_dec3 = [V_mag(3),V_pha(3)];            % Make a starting guess at the solution
    options = optimset('Algorithm','active-set', 'TolFun', 1e-16, 'MaxFunEvals', 2000);
    [V,fval] = fmincon(@obj_fun_mic_dec3,V0_mic_dec3,[],[],[],[],[],[],@con_fun_mic_dec3,options);
    V_temp = V;
    save('obj_mic3_V_temp.mat','V_temp');
    u_temp = V_mag;
    save('obj_mic3_u_temp.mat','u_temp');
    change = 1;
else
    load obj_mic3_V_temp
    V = V_temp;
    change = 0;
end

V_vector(3) = V(1)*exp(1j*V(2));
S_opt = 3*diag(V_vector(3))*conj(Y_bus_mcm*V_vector);
P_opt_mic_dec3 = real(S_opt); Q_opt_mic_dec3 = imag(S_opt);
V_mag_dec3 = V(1);
V_pha_dec3 = V(2);

sp_temp = [P_opt_mic_dec3, Q_opt_mic_dec3];
save('sp_mic3_temp.mat','sp_temp');    

sys = [change,P_opt_mic_dec3, Q_opt_mic_dec3, V_mag_dec3, V_pha_dec3];

