function [sys,x0,str,ts] = opt_solver_obj_dec2_snn(t,x,u,flag)
%determine set points of microsources within microgrid 1
switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0,
    [sys,x0,str,ts]=mdlInitializeSizes();

  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  case 3,
    sys=mdlOutputs(t,x,u);

  %%%%%%%%%%%%%%%%%%%
  % Unhandled flags %
  %%%%%%%%%%%%%%%%%%%
  case { 1, 2, 4, 9 },
    sys = [];

  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise
    DAStudio.error('Simulink:blocks:unhandledFlag', num2str(flag));

end
% end csfunc

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes()

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 9;
sizes.NumInputs      = 10;
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
x0  = [];
str = [];
ts  = [-1 0];

% end mdlInitializeSizes
%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)
global P_req_mic Q_req_mic E_req_mic Pha_req_mic switch_mg1
    recal = u(1);
    P_req_mic = u(2);
    Q_req_mic = u(3);
    E_req_mic = u(4);
    Pha_req_mic = u(5);
    switch_mg1 = u(6:9);
    trig_mg1 = u(10);
    
[Y_bus_mic, G_bus_mic, B_bus_mic, P_u_mic, P_l_mic, Q_u_mic, Q_l_mic, V_u_mic, V_l_mic, P_load_mic, Q_load_mic] = model_ini_mg1(switch_mg1);

%centralized optimization problem
if ((recal == 1)||(trig_mg1 == 1))
    V0_mic = [1,1,1,0,0,0];
    options = optimset('Algorithm','active-set', 'TolFun', 1e-16, 'MaxFunEvals', 2000);
    [V,fval] = fmincon(@obj_fun_obj_dec2,V0_mic,[],[],[],[],[],[],@con_fun_obj_dec2,options);
    V_temp = V;
    save('obj_dec2_V_temp.mat','V_temp');
else
    load obj_dec2_V_temp
    V = V_temp;
end
    
V_mag = [E_req_mic V(1) V(2) V(3)]; V_pha = [Pha_req_mic V(4) V(5) V(6)];
V_vector_mic = [V_mag(1)*exp(1j*V_pha(1)); V_mag(2)*exp(1j*V_pha(2)); V_mag(3)*exp(1j*V_pha(3)); V_mag(4)*exp(1j*V_pha(4))];
S_opt_mic = 3*diag(V_vector_mic)*conj(Y_bus_mic*V_vector_mic);
P_opt_mic = real(S_opt_mic)'+[0, 3*P_load_mic(1,2:4)*diag([V_mag(2)^2, V_mag(3)^2, V_mag(4)^2])]; 
Q_opt_mic = imag(S_opt_mic)'+[0, 3*Q_load_mic(1,2:4)*diag([V_mag(2)^2, V_mag(3)^2, V_mag(4)^2])];
V_req_mic = [0, V_mag(2:4)]+0.05*[0, Q_opt_mic(2:4)];

sys = [P_opt_mic(2:4), V_req_mic(2:4), Q_opt_mic(2:4)];
